﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System;
using System.Collections.Generic;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using BlobStoreApi;

namespace BlobStore
{
    public partial class Login : UserControl
    {
        private const string ServiceSetting = "Service";
        private const string AccountNameSetting = "AccountName";
        private const string AccountKeySetting = "AccountKey";
        private const string StoreSetting = "Store";

        private LoginView _loginView = new LoginView();

        public BlobStoreClient BlobStore { get; set; }

        public EventHandler<EventArgs> LoginCompleted;

        public Login()
        {
            InitializeComponent();

            _loginView.LoginCompleted = blobStoreClient =>
            {
                // Capture login info
                BlobStore = blobStoreClient;
                SetApplicationSetting(ServiceSetting, _loginView.Service);
                SetApplicationSetting(AccountNameSetting, _loginView.AccountName);
                SetApplicationSetting(AccountKeySetting, _loginView.AccountKey);
                SetApplicationSetting(StoreSetting, _loginView.Store);

                // Invoke event
                var handler = LoginCompleted;
                if (null != handler)
                {
                    handler.Invoke(this, EventArgs.Empty);
                }
            };

            // Customize TextBox and PasswordBox to update their Bindings for each key
            // and to "submit" when Enter is pressed
            foreach (var textBox in ControlContainer.Children.OfType<TextBox>())
            {
                var capturedTextBox = textBox;
                capturedTextBox.TextChanged += delegate(object sender, TextChangedEventArgs e)
                {
                    capturedTextBox.GetBindingExpression(TextBox.TextProperty).UpdateSource();
                };
                capturedTextBox.KeyDown += delegate(object sender, KeyEventArgs e)
                {
                    if (Key.Enter == e.Key)
                    {
                        if (_loginView.LoginCommand.CanExecute(null))
                        {
                            _loginView.LoginCommand.Execute(null);
                        }
                    }
                };
            }
            foreach (var passwordBox in ControlContainer.Children.OfType<PasswordBox>())
            {
                var capturedPasswordBox = passwordBox;
                capturedPasswordBox.PasswordChanged += delegate(object sender, RoutedEventArgs e)
                {
                    capturedPasswordBox.GetBindingExpression(PasswordBox.PasswordProperty).UpdateSource();
                };
                capturedPasswordBox.KeyDown += delegate(object sender, KeyEventArgs e)
                {
                    if (Key.Enter == e.Key)
                    {
                        if (_loginView.LoginCommand.CanExecute(null))
                        {
                            _loginView.LoginCommand.Execute(null);
                        }
                    }
                };
            }

            Root.DataContext = _loginView;

            // Restore saved login info (if possible)
            try
            {
                _loginView.Service = GetApplicationSetting<ServiceType>(ServiceSetting);
                _loginView.AccountName = GetApplicationSetting<string>(AccountNameSetting);
                _loginView.AccountKey = GetApplicationSetting<string>(AccountKeySetting);
                _loginView.Store = GetApplicationSetting<string>(StoreSetting);
            }
            catch (KeyNotFoundException)
            {
                // Ignore
            }
            catch (IsolatedStorageException)
            {
                // Ignore
            }

            // Set default focus to Login button
            Loaded += delegate
            {
                ControlContainer.Children.OfType<Button>().Single().Focus();
            };
        }

        public void ShowControl(bool show)
        {
            if (show)
            {
                throw new NotSupportedException();
            }
            else
            {
                VisualStateManager.GoToState(this, "Hide", true);
            }
        }

        private T GetApplicationSetting<T>(string settingName)
        {
            try
            {
                return (T)IsolatedStorageSettings.ApplicationSettings[settingName];
            }
            catch (KeyNotFoundException)
            {
                return default(T);
            }
        }

        private void SetApplicationSetting(string settingName, object settingValue)
        {
            IsolatedStorageSettings.ApplicationSettings[settingName] = settingValue;
        }
    }

    public enum ServiceType { IsolatedStorage = 0, WindowsAzure = 1, AmazonS3 = 2 };

    public class LoginView : NotifyPropertyChangedBase
    {
        public Action<BlobStoreClient> LoginCompleted { get; set; }

        public LoginView()
        {
            // Using ComboBoxItem here is atypical - but convenient
            _services = new List<ComboBoxItem>
            {
                new ComboBoxItem { Content = "Isolated Storage", Tag = ServiceType.IsolatedStorage },
                new ComboBoxItem { Content = "Windows Azure", Tag = ServiceType.WindowsAzure },
                new ComboBoxItem { Content = "Amazon S3", Tag = ServiceType.AmazonS3 },
            };

            // Update state of Login button according to available data
            _loginCommand = new DelegateCommand
            {
                CanExecuteDelegate = (p) =>
                {
                    return !string.IsNullOrEmpty(AccountName) && !string.IsNullOrEmpty(AccountKey) && (!StoreEnabled || !string.IsNullOrEmpty(Store));
                },
                ExecuteDelegate = (p) =>
                {
                    BlobStoreClient blobStoreClient;
                    switch (Service)
                    {
                        case ServiceType.IsolatedStorage:
                            blobStoreClient = new IsolatedStorageBlobStoreClient();
                            break;
                        case ServiceType.WindowsAzure:
                            blobStoreClient = new AzureBlobStoreClient(AccountName, AccountKey, Store);
                            break;
                        case ServiceType.AmazonS3:
                            blobStoreClient = new S3BlobStoreClient(AccountName, AccountKey, Store);
                            break;
                        default:
                            throw new NotSupportedException();
                    }
                    LoginCompleted(blobStoreClient);
                },
            };
        }

        public IEnumerable<ComboBoxItem> Services
        {
            get { return _services; }
        }
        private List<ComboBoxItem> _services;

        public int ServiceIndex
        {
            get
            {
                return _services
                    .IndexOf(_services.Where(i => (ServiceType)i.Tag == Service)
                    .Single());
            }
            set
            {
                Service = (ServiceType)_services[value].Tag;
            }
        }

        public ServiceType Service
        {
            get { return _serviceType; }
            set
            {
                _serviceType = value;
                InvokePropertyChanged("ServiceType");
                InvokePropertyChanged("ServiceIndex");
                InvokePropertyChanged("Store");
                InvokePropertyChanged("StoreEnabled");
                InvokePropertyChanged("StoreLabelBrush");
            }
        }
        private ServiceType _serviceType;

        public string AccountName
        {
            get { return _accountName; }
            set
            {
                _accountName = value;
                InvokePropertyChanged("AccountName");
                _loginCommand.InvokeCanExecuteChanged();
            }
        }
        private string _accountName;

        public string AccountKey
        {
            get { return _accountKey; }
            set
            {
                _accountKey = value;
                InvokePropertyChanged("AccountKey");
                _loginCommand.InvokeCanExecuteChanged();
            }
        }
        private string _accountKey;

        public string Store
        {
            get { return StoreEnabled ? _store : ""; }
            set
            {
                _store = value;
                InvokePropertyChanged("Store");
                _loginCommand.InvokeCanExecuteChanged();
            }
        }
        private string _store;

        public bool StoreEnabled
        {
            get { return ServiceType.IsolatedStorage != Service; }
        }

        public object StoreLabelBrush
        {
            get { return StoreEnabled ? DependencyProperty.UnsetValue : new SolidColorBrush(Colors.Gray); }
        }

        public ICommand LoginCommand
        {
            get { return _loginCommand; }
        }
        private DelegateCommand _loginCommand;
    }
}
