﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Security;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Threading;
using BlobStoreApi;

namespace BlobStore
{
    public partial class MainPage : UserControl
    {
        private MainPageView _mainPageView = new MainPageView();

        public MainPage()
        {
            InitializeComponent();

            // Hook up Progress/Login here to work around Visibility=Collapsed design-time limitation
            var progress = new Progress();
            progress.SetBinding(Progress.ProgressCurrentProperty, new Binding("ProgressCurrent"));
            progress.SetBinding(Progress.ProgressTotalProperty, new Binding("ProgressTotal"));
            progress.SetBinding(Progress.MessageProperty, new Binding("Message"));
            OverlayContainer.Children.Add(progress);

            var login = new Login();
            login.LoginCompleted += delegate
            {
                _mainPageView.BlobStoreClient = login.BlobStore;
                _mainPageView.InterfaceEnabled = true;
                login.ShowControl(false);
                AllowDrop = true;
                Update();
            };
            OverlayContainer.Children.Add(login);

            DataContext = _mainPageView;
        }

        private void Update()
        {
            _mainPageView.Message = "Updating...";
            _mainPageView.ProgressTotal = 0;
            _mainPageView.Blobs.Clear();
            _mainPageView.BlobStoreClient.GetBlobInfos(blobInfos =>
            {
                Dispatcher.BeginInvoke(() =>
                {
                    foreach (var blobInfo in blobInfos.OrderBy(bi => bi.Name))
                    {
                        _mainPageView.Blobs.Add(blobInfo);
                    }
                    _mainPageView.ProgressTotal = -1;
                });
            },
            (ex) => { throw ex; });
        }

        protected override void OnDrop(DragEventArgs e)
        {
            base.OnDrop(e);
            var fileDropFileInfos = e.Data.GetData(DataFormats.FileDrop) as FileInfo[];
            if (null != fileDropFileInfos)
            {
                UploadNextFileInfo(fileDropFileInfos.ToList());
            }
        }

        private void UploadNextFileInfo(List<FileInfo> fileInfos)
        {
            var fileInfo = fileInfos[0];
            fileInfos.RemoveAt(0);
            _mainPageView.Message = string.Format(CultureInfo.CurrentCulture, "Transferring {0}...", fileInfo.Name);
            _mainPageView.ProgressTotal = fileInfo.Length;
            var fileStream = fileInfo.OpenRead();
            _mainPageView.BlobStoreClient.PutBlob(
                new BlobInfo(fileInfo.Name),
                fileInfo.Length,
                blobStream =>
                {
                    var buffer = new byte[1024];
                    long totalBytesRead = 0;
                    var bytesRead = 0;
                    while (0 < (bytesRead = fileStream.Read(buffer, 0, buffer.Length)))
                    {
                        blobStream.Write(buffer, 0, bytesRead);
                        totalBytesRead += bytesRead;
                        Dispatcher.BeginInvoke(() => _mainPageView.ProgressCurrent = totalBytesRead);
                    }
                },
                () =>
                {
                    fileStream.Close();
                    Dispatcher.BeginInvoke(() =>
                    {
                        if (0 == fileInfos.Count)
                        {
                            _mainPageView.ProgressTotal = -1;
                            Update();
                        }
                        else
                        {
                            UploadNextFileInfo(fileInfos);
                        }
                    });
                },
                (ex) => { throw ex; });
        }

        private void Blob_DeleteClicked(object sender, EventArgs e)
        {
            var blobInfo = ((Blob)sender).BlobInfo;
            _mainPageView.Message = string.Format(CultureInfo.CurrentCulture, "Deleting {0}...", blobInfo.Name);
            _mainPageView.ProgressTotal = 0;
            _mainPageView.BlobStoreClient.DeleteBlob(
                blobInfo,
                () =>
                {
                    Dispatcher.BeginInvoke(() =>
                    {
                        _mainPageView.ProgressTotal = -1;
                        Update();
                    });
                },
                (ex) => { throw ex; });
        }

        private void Blob_RightClicked(object sender, EventArgs e)
        {
            var blobInfo = ((Blob)sender).BlobInfo;
            try
            {
                Clipboard.SetText(blobInfo.FriendlyUri.ToString());
            }
            catch (SecurityException)
            {
                // User rejected clipboard access request
            }
        }

        private void Refresh_Click(object sender, RoutedEventArgs e)
        {
            Update();
        }

        public void DisplayException(Exception e)
        {
            // Clear UI and show simple error message
            // Usually a networking issue; application can't continue
            Root.Children.Clear();
            var errorPanel = new Grid { Background = new SolidColorBrush(Color.FromArgb(255, 255, 200, 200)) };
            errorPanel.Children.Add(new TextBlock
            {
                Text = "Sorry, the web service appears to be having trouble.\nPlease try again later...",
                FontSize = 16,
                HorizontalAlignment= HorizontalAlignment.Center,
                VerticalAlignment = VerticalAlignment.Center,
            });
            Root.Children.Add(errorPanel);
        }
    }

    public class MainPageView : NotifyPropertyChangedBase
    {
        public MainPageView()
        {
            Blobs = new ObservableCollection<BlobInfo>();
            ProgressTotal = -1;
        }

        public ObservableCollection<BlobInfo> Blobs { get; private set; }

        public BlobStoreClient BlobStoreClient { get; set; }

        public long ProgressTotal
        {
            get { return _progressTotal; }
            set
            {
                _progressTotal = value;
                InvokePropertyChanged("ProgressTotal");
            }
        }
        private long _progressTotal;

        public long ProgressCurrent
        {
            get { return _progressCurrent; }
            set
            {
                _progressCurrent = value;
                InvokePropertyChanged("ProgressCurrent");
            }
        }
        private long _progressCurrent;

        public string Message
        {
            get { return _message; }
            set
            {
                _message = value;
                InvokePropertyChanged("Message");
            }
        }
        private string _message;

        public bool InterfaceEnabled
        {
            get { return _interfaceEnabled; }
            set
            {
                _interfaceEnabled = value;
                InvokePropertyChanged("InterfaceEnabled");
            }
        }
        private bool _interfaceEnabled;
    }

#if DEBUG
    public class DesignTimeMainPageView : MainPageView
    {
        public DesignTimeMainPageView()
        {
            Blobs.Add(new BlobInfo("File.txt"));
            Blobs.Add(new BlobInfo("File with longer name.txt"));
            Blobs.Add(new BlobInfo("program.exe"));
            Blobs.Add(new BlobInfo("File with an extremely long name that is intended to help test wrapping.txt"));
        }
    }
#endif
}
