﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Text;
using System.Windows;
using BlobStoreApi;
using Microsoft.Phone.Controls;

namespace BlobStoreOnWindowsPhone
{
    public partial class MainPage : PhoneApplicationPage
    {
        // TODO: Switch to AzureBlobStoreClient or S3BlobStoreClient after entering your connection information
        private readonly BlobStoreClient _client = new IsolatedStorageBlobStoreClient();
        //private readonly BlobStoreClient _client = new AzureBlobStoreClient("Account Name", "Access Key", "Container Name");
        //private readonly BlobStoreClient _client = new S3BlobStoreClient("Access Key ID", "Secret Access Key", "Bucket Name");

        private readonly Model _model;

        public MainPage()
        {
            InitializeComponent();
            _model = new Model(LoadNoteMessage);
            DataContext = _model;
            LoadNotes();
        }

        private void LoadNotes()
        {
            _model.StatusMessage = "Querying...";
            _model.Notes.Clear();
            _client.GetBlobInfos((blobInfos) =>
            {
                Dispatcher.BeginInvoke(() =>
                {
                    foreach (var blobInfo in blobInfos)
                    {
                        long ticks;
                        if (long.TryParse(blobInfo.Name, out ticks))
                        {
                            _model.Notes.Add(new Note(new DateTime(ticks).ToString("F"), blobInfo));
                        }
                    }
                    _model.StatusMessage = null;
                });
            },
            (ex) => { throw ex; });
        }

        private void LoadNoteMessage(Note note, Action completed)
        {
            _model.StatusMessage = "Loading...";
            var bytes = new List<byte>();
            _client.GetBlob(
                note.BlobInfo,
                (stream) =>
                {
                    var b = stream.ReadByte();
                    while (-1 != b)
                    {
                        bytes.Add((byte)b);
                        b = stream.ReadByte();
                    }
                },
                () => Dispatcher.BeginInvoke(() =>
                {
                    note.Message = Encoding.UTF8.GetString(bytes.ToArray(), 0, bytes.Count);
                    _model.StatusMessage = null;
                    completed();
                }),
                (ex) => { throw ex; });
        }

        private void HandleAddNoteButtonClick(object sender, RoutedEventArgs e)
        {
            _model.StatusMessage = "Writing...";
            var message = string.IsNullOrEmpty(NewNoteTextBox.Text) ? "[blank]" : NewNoteTextBox.Text;
            var bytes = Encoding.UTF8.GetBytes(message);
            NewNoteTextBox.Text = "";
            var now = DateTime.Now;
            var blobInfo = new BlobInfo(now.Ticks.ToString());
            _client.PutBlob(
                blobInfo,
                bytes.Length,
                (stream) => stream.Write(bytes, 0, bytes.Length),
                () => Dispatcher.BeginInvoke(() =>
                {
                    _model.Notes.Add(new Note(now.ToString("F"), blobInfo, message));
                    _model.StatusMessage = null;
                }),
                (ex) => { throw ex; });
        }

        private void HandleDeleteNoteButtonClick(object sender, RoutedEventArgs e)
        {
            _model.StatusMessage = "Deleting...";
            var note = _model.Notes[_model.SelectedNoteIndex];
            _client.DeleteBlob(
                note.BlobInfo,
                () => Dispatcher.BeginInvoke(() =>
                {
                    _model.Notes.Remove(note);
                    _model.StatusMessage = null;
                }),
                (ex) => { throw ex; });
        }
    }

    public class Model : INotifyPropertyChanged
    {
        public IList<Note> Notes { get { return _notes; } }
        public Action<Note, Action> LoadNoteMessage { get; private set; }

        private readonly ObservableCollection<Note> _notes = new ObservableCollection<Note>();

        public Model(Action<Note, Action> loadNoteMessage)
        {
            LoadNoteMessage = loadNoteMessage;
            SelectedNoteIndex = -1;
            StatusVisibility = Visibility.Collapsed;
        }

        public string StatusMessage
        {
            get { return _statusMessage; }
            set
            {
                _statusMessage = value;
                InvokePropertyChanged("StatusMessage");
                StatusVisibility = (null == _statusMessage) ? Visibility.Collapsed : Visibility.Visible;
                InvokePropertyChanged("StatusVisibility");
            }
        }
        private string _statusMessage;

        public Visibility StatusVisibility { get; private set; }

        public int SelectedNoteIndex
        {
            get { return _selectedNoteIndex; }
            set
            {
                _selectedNoteIndex = value;
                DeleteNoteEnabled = -1 != _selectedNoteIndex;
                if (-1 == _selectedNoteIndex)
                {
                    SelectedNoteMessage = "[no note selected]";
                }
                else
                {
                    if (null == _notes[_selectedNoteIndex].Message)
                    {
                        LoadNoteMessage(_notes[_selectedNoteIndex], () => SelectedNoteMessage = _notes[_selectedNoteIndex].Message);
                    }
                    else
                    {
                        SelectedNoteMessage = _notes[_selectedNoteIndex].Message;
                    }
                }
            }
        }
        private int _selectedNoteIndex;

        public string SelectedNoteMessage
        {
            get { return _selectedNoteMessage; }
            set
            {
                _selectedNoteMessage = value;
                InvokePropertyChanged("SelectedNoteMessage");
            }
        }
        private string _selectedNoteMessage;

        public bool DeleteNoteEnabled
        {
            get { return _deleteNoteEnabled; }
            set
            {
                _deleteNoteEnabled = value;
                InvokePropertyChanged("DeleteNoteEnabled");
            }
        }
        private bool _deleteNoteEnabled;

        private void InvokePropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class Note
    {
        public string Title { get; private set; }
        public BlobInfo BlobInfo { get; private set; }
        public string Message { get; set; }

        public Note(string title,  BlobInfo blobInfo)
        {
            Title = title;
            BlobInfo = blobInfo;
        }

        public Note(string title, BlobInfo blobInfo, string message)
            : this(title, blobInfo)
        {
            Message = message;
        }
    }
}
